unit
  TTimer;

interface

uses
  Windows,Classes;
{------------------------------------------------------------------------------}

type
  TTimerThread = class(TThread)
  private
    FOwner: TComponent;
  protected
    procedure Execute; override;
  public
    FTerminate: Boolean;
    constructor Create(AOwner: TComponent);
  end;

  TThreadTimer = class(TComponent)
  private
    FEnabled: Boolean;
    FInterval: Cardinal;
    FOnTimer: TNotifyEvent;
    FTimerThread: TTimerThread;
    FThreadPriority: TThreadPriority;
    procedure SetEnabled(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetThreadPriority(Value: TThreadPriority);
  protected
    procedure Timer; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property Interval: Cardinal read FInterval write SetInterval default 1000;
    property ThreadPriority: TThreadPriority read FThreadPriority write SetThreadPriority default tpNormal;
    property OnTimer: TNotifyEvent read FOnTimer write FOnTimer;
  end;
{------------------------------------------------------------------------------}

procedure Register;

implementation

uses
  Forms,Consts;
{------------------------------------------------------------------------------}

constructor TTimerThread.Create(AOwner: TComponent);
begin
  inherited Create(False);
  FOwner:=AOwner;
end;
{------------------------------------------------------------------------------}

procedure TTimerThread.Execute;
begin
  Priority:=(FOwner as TThreadTimer).FThreadPriority;
  repeat
    if not FTerminate and (SleepEx((FOwner as TThreadTimer).Interval,False) = 0) 
      then Synchronize((FOwner as TThreadTimer).Timer);
  until FTerminate;
end;
{------------------------------------------------------------------------------}

constructor TThreadTimer.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled:=True;
  FInterval:=1000;
  FThreadPriority:=tpNormal;
  FTimerThread:=TTimerThread.Create(Self);
end;
{------------------------------------------------------------------------------}

destructor TThreadTimer.Destroy;
begin
  Enabled:=False;
  FOnTimer:=nil;
  if Assigned(FTimerThread) then
  begin
    FTimerThread.FTerminate:=True;
    FTimerThread.Terminate;
    FTimerThread.Free;
  end;
  inherited Destroy;
end;
{------------------------------------------------------------------------------}

procedure TThreadTimer.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled:=Value;
    if Assigned(FTimerThread) then
      if FEnabled then FTimerThread.Suspend else FTimerThread.Resume;
  end;    
end;
{------------------------------------------------------------------------------}

procedure TThreadTimer.SetInterval(Value: Cardinal);
begin
  if Value <> FInterval then FInterval:=Value;
end;
{------------------------------------------------------------------------------}

procedure TThreadTimer.SetThreadPriority(Value: TThreadPriority);
begin
  if Value <> FThreadPriority then
  begin
    FThreadPriority:=Value;
    if Assigned(FTimerThread) then FTimerThread.Priority:=Value;
  end;
end;
{------------------------------------------------------------------------------}

procedure TThreadTimer.Timer;
begin
  if FEnabled and Assigned(FOnTimer) then FOnTimer(Self);
end;
{------------------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents('Udo',[TThreadTimer]);
end;
{------------------------------------------------------------------------------}

initialization
end.
